import json

import wagtail
from django.templatetags.static import static
from django.urls import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from wagtail import hooks
from wagtail.admin.action_menu import ActionMenuItem
from wagtail.snippets.models import register_snippet

from wagtaildraftsharing.actions import (
    register_wagtaildraftsharing_log_actions,
)
from wagtaildraftsharing.snippets import WagtaildraftsharingLinkSnippetViewSet

from .settings import settings as draftsharing_settings

register_snippet(WagtaildraftsharingLinkSnippetViewSet)

hooks.register("register_log_actions")(register_wagtaildraftsharing_log_actions)


@hooks.register("insert_global_admin_js")
def editor_js():
    # We need different approaches to the JS depending on whether it's
    # Wagtail 5 (vanilla JS) or Wagtail 6 (Stimulus)
    if int(wagtail.__version__[0]) < 6:
        SHARING_JS_TEMPLATE = """<script src="{}"></script>"""
        SHARING_JS_PATH = "wagtaildraftsharing/js/wagtaildraftsharing.js"
    else:
        SHARING_JS_TEMPLATE = """<script type="module" src="{}"></script>"""
        SHARING_JS_PATH = "wagtaildraftsharing/js/wagtaildraftsharing_controller.js"

    return format_html(
        SHARING_JS_TEMPLATE,
        static(SHARING_JS_PATH),
    ) + mark_safe(
        f"""
        <script id="wagtaildraftsharing-config" type="application/json">
            {json.dumps({
                'urls': {
                    'create': reverse('wagtaildraftsharing:create'),
                },
            })}
        </script>"""
    )


class DraftsharingPageActionMenuItem(ActionMenuItem):
    # Adds a button for sharing a draft to the bottom-of-page action menu
    # when a page is being edited, but only if there is a saved draft to share
    order = 1600
    name = "action-draftsharing"
    icon_name = "view"
    label = draftsharing_settings.MENU_ITEM_LABEL

    template_name = "wagtaildraftsharing/action_menu_item.html"

    def get_context_data(self, parent_context):
        context_data = super().get_context_data(parent_context)

        context_data["revision"] = context_data["page"].latest_revision
        return context_data

    def is_shown(self, context):
        return "page" in context and context["page"].has_unpublished_changes


@hooks.register("register_page_action_menu_item")
def register_draftsharing_page_action_menu_item():
    return DraftsharingPageActionMenuItem()



from wagtail.admin.panels import FieldPanel, ObjectList
from wagtail.snippets.views.snippets import SnippetViewSet

from wagtaildraftsharing.models import WagtaildraftsharingLink

from .settings import settings as draftsharing_settings


class WagtaildraftsharingLinkSnippetViewSet(SnippetViewSet):
    model = WagtaildraftsharingLink
    base_url_path = "wagtaildraftsharing"
    menu_icon = "view"
    menu_order = draftsharing_settings.ADMIN_MENU_POSITION
    add_to_settings_menu = False
    add_to_admin_menu = True
    list_display = ("__str__", "is_active", "created_by", "share_url")
    list_filter = ("is_active",)

    edit_handler = ObjectList([
        FieldPanel("revision", read_only=True),
        FieldPanel(
            "is_active",
            help_text=(
                "When false, the sharing link will not be viewable."
            )
        ),
        FieldPanel(
            "active_until",
            help_text=(
                "The link will not be viewable after this date. "
                "Leave blank if the link should never expire."
            )
        ),
    ])


    def get_queryset(self, request):
        return WagtaildraftsharingLink.objects.all().prefetch_related(
            "revision", "revision__content_object", "created_by"
        )



import dataclasses
from typing import cast

from django.conf import settings as django_settings
from django.utils.functional import SimpleLazyObject

_DEFAULT_MAX_TTL = 28 * 24 * 60 * 60  # 28 days


@dataclasses.dataclass
class WagtaildraftsharingSettings:
    ADMIN_MENU_POSITION: int = 200
    VERBOSE_NAME: str = "Draftsharing Link"
    VERBOSE_NAME_PLURAL: str = "Draftsharing Links"
    MENU_ITEM_LABEL: str = "Create draft sharing link"
    MAX_TTL: int = _DEFAULT_MAX_TTL


def _init_settings():
    """
    Get and validate Wagtaildraftsharing settings from the Django settings.
    """

    setting_name = "WAGTAILDRAFTSHARING"
    django_settings_dict = getattr(django_settings, setting_name, {})

    overriden_defaults = {}

    for optional_setting_name in [
        "ADMIN_MENU_POSITION",
        "VERBOSE_NAME",
        "VERBOSE_NAME_PLURAL",
        "MENU_ITEM_LABEL",
        "MAX_TTL",
    ]:
        if optional_setting_name in django_settings_dict:
            overriden_defaults[optional_setting_name] = django_settings_dict[
                optional_setting_name
            ]

    return WagtaildraftsharingSettings(**overriden_defaults)


settings = cast(WagtaildraftsharingSettings, SimpleLazyObject(_init_settings))



import json
from unittest.mock import Mock, patch

import wagtail
from django.test import RequestFactory, TestCase
from django.urls import reverse

from wagtaildraftsharing import wagtail_hooks


class WagtailHooksTests(TestCase):
    def test_editor_js(self):
        self.maxDiff = None
        for wagtail_version, expected in (
            (
                "5.0.0",  # only major version matters
                # Whitespace here is horrible. TODO: improve
                f"""<script src="/static/wagtaildraftsharing/js/wagtaildraftsharing.js"></script>
        <script id="wagtaildraftsharing-config" type="application/json">
            {json.dumps({
                'urls': {
                    'create': reverse('wagtaildraftsharing:create'),
                },
            })}
        </script>""",  # NOQA: E501
            ),
            (
                "6.0.0",  # only major version matters
                # Whitespace here is horrible. TODO: improve
                f"""<script type="module" src="/static/wagtaildraftsharing/js/wagtaildraftsharing_controller.js"></script>
        <script id="wagtaildraftsharing-config" type="application/json">
            {json.dumps({
                'urls': {
                    'create': reverse('wagtaildraftsharing:create'),
                },
            })}
        </script>""",  # NOQA: E501
            ),
        ):
            with self.subTest(wagtail_version=wagtail_version, expected=expected):
                with patch.object(wagtail, "__version__", wagtail_version):
                    js_tags = wagtail_hooks.editor_js()
                    self.assertEqual(js_tags, expected)

    def test_action_menu_hook__get_context_data(self):
        req = RequestFactory().get("/path/is/not/relevant/")

        mock_revision = Mock(name="mock_revision")

        mock_page = Mock(name="mock_page")
        mock_page.latest_revision = mock_revision
        menu_item = wagtail_hooks.DraftsharingPageActionMenuItem()

        self.assertEqual(
            menu_item.get_context_data(
                parent_context={"request": req, "page": mock_page}
            ),
            {
                "classname": "",
                "icon_name": "view",
                "label": "Create draft sharing link",
                "name": "action-draftsharing",
                "page": mock_page,
                "request": req,
                "revision": mock_revision,
                "url": None,
            },
        )

    def test_action_menu_hook__is_shown(self):
        mock_page = Mock(name="mock_page")
        menu_item = wagtail_hooks.DraftsharingPageActionMenuItem()

        mock_page.has_unpublished_changes = False
        self.assertFalse(menu_item.is_shown({"page": mock_page}))

        mock_page.has_unpublished_changes = True
        self.assertTrue(menu_item.is_shown({"page": mock_page}))

    def test_action_menu_hook__is_shown__no_page_in_context(self):
        menu_item = wagtail_hooks.DraftsharingPageActionMenuItem()
        self.assertFalse(menu_item.is_shown({}))
        self.assertFalse(menu_item.is_shown({"something_that_is_not_a_page": "here"}))



import uuid

import wagtail
from django.conf import settings
from django.db import models
from django.urls import reverse
from django.utils.html import format_html
from django.utils.timezone import timedelta
from wagtail.log_actions import log

from wagtaildraftsharing.actions import (
    WAGTAILDRAFTSHARING_CREATE_SHARING_LINK,
    WAGTAILDRAFTSHARING_REUSE_SHARING_LINK,
)
from wagtaildraftsharing.utils import tz_aware_utc_now

from .settings import settings as draftsharing_settings


class WagtaildraftsharingLinkManager(models.Manager):
    def _get_active_until(self, max_ttl):
        if max_ttl is None:
            max_ttl = draftsharing_settings.MAX_TTL
        if max_ttl > 0:
            active_until = tz_aware_utc_now() + timedelta(seconds=max_ttl)
        else:
            active_until = None

        return active_until

    def create_for_revision(self, *, revision, user, max_ttl=None):
        # Always creates a new sharing link
        sharing_link = WagtaildraftsharingLink.objects.create(
            revision=revision,
            key=uuid.uuid4(),
            created_by=user,
            active_until=self._get_active_until(max_ttl=max_ttl),
        )
        log(
            instance=revision.content_object,
            action=WAGTAILDRAFTSHARING_CREATE_SHARING_LINK,
            user=user,
            revision=revision,
            data={"revision": revision.id},
        )

        return sharing_link

    def get_or_create_for_revision(self, *, revision, user, max_ttl=None):
        """
        Deprecated: prefer self.create_for_revision() instead

        Creates a new sharing link if it doesn't exist, else returns an
        existing one. Note that this may give you a link that is soon to
        expire if it was made close to max_ttl seconds ago...
        """

        active_until = self._get_active_until(max_ttl=max_ttl)

        sharing_link, created = WagtaildraftsharingLink.objects.get_or_create(
            revision=revision,
            defaults={
                "key": uuid.uuid4(),
                "created_by": user,
                "active_until": active_until,
            },
        )
        if created:
            log(
                instance=revision.content_object,
                action=WAGTAILDRAFTSHARING_CREATE_SHARING_LINK,
                user=user,
                revision=revision,
                data={"revision": revision.id},
            )
        else:
            log(
                instance=revision.content_object,
                action=WAGTAILDRAFTSHARING_REUSE_SHARING_LINK,
                user=user,
                revision=revision,
                data={"revision": revision.id},
            )

        return sharing_link


class WagtaildraftsharingLink(models.Model):
    key = models.UUIDField(
        unique=True,
        default=uuid.uuid4,
        editable=False,
        primary_key=False,
    )
    revision = models.ForeignKey(
        "wagtailcore.Revision",
        on_delete=models.CASCADE,
        related_name="+",
    )
    is_active = models.BooleanField(default=True)
    active_until = models.DateTimeField(
        null=True,
        blank=True,
        help_text="Leave blank for no expiry",
    )
    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name="+",
        editable=False,
    )

    objects = WagtaildraftsharingLinkManager()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Set the verbose names from settings,
        # in a way that doesn't trigger migrations
        self._meta.verbose_name = draftsharing_settings.VERBOSE_NAME
        self._meta.verbose_name_plural = draftsharing_settings.VERBOSE_NAME_PLURAL

    def __str__(self):
        return f"Revision {self.revision_id} of {self.revision.content_object}"

    class Meta:
        # These may be changed in __init__ from settings
        verbose_name = "Sharing Link"
        verbose_name_plural = "Sharing Links"

    @property
    def url(self):
        return reverse("wagtaildraftsharing:view", kwargs={"key": self.key})

    @property
    def share_url(self):
        # Make the existing link easily shareable.
        # Also note that the View button is changed into a "Copy" button via JS
        if int(wagtail.__version__[0]) < 6:
            template = """<a
                class="button button-secondary button-small"
                data-wagtaildraftsharing-url
                target="_blank"
                rel="noopener noreferrer"
                href="{}">View</a>"""
        else:
            template = """<a
                class="button button-secondary button-small"
                data-controller="wagtaildraftsharing"
                data-wagtaildraftsharing-snippet-url
                target="_blank"
                rel="noopener noreferrer"
                href="{}">View</a>"""

        return format_html(template, self.url)



from django.forms import ModelForm

from wagtaildraftsharing.models import WagtaildraftsharingLink


class CreateWagtaildraftsharingLinkForm(ModelForm):
    class Meta:
        model = WagtaildraftsharingLink
        fields = ["revision"]

    def clean(self):
        # Since we are using get_or_create, we don't need to check for uniqueness
        return



from django.conf import settings
from django.urls import include, path

from wagtail.admin import urls as wagtailadmin_urls

import wagtaildraftsharing.urls as wagtaildraftsharing_urls


urlpatterns = [
    path("admin/", include(wagtailadmin_urls)),
    path("wagtaildraftsharing/", include(wagtaildraftsharing_urls)),
]


if settings.DEBUG:
    from django.conf.urls.static import static
    from django.contrib.staticfiles.urls import staticfiles_urlpatterns

    urlpatterns += staticfiles_urlpatterns()
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



from datetime import timezone

from django.utils.timezone import is_aware, make_aware
from django.utils.timezone import now as timezone_now


def tz_aware_utc_now():
    now = timezone_now()
    # Depending on your version of Django and/or setting.TZ_NOW, timezone_now()
    # may not actually be TZ aware, but we always want it to be for these links
    if not is_aware(now):
        now = make_aware(now, timezone.utc)
    return now



import datetime
from textwrap import dedent
from unittest.mock import patch

import wagtail
from django.contrib.auth.models import User
from django.test import TestCase
from django.utils.timezone import is_aware
from freezegun import freeze_time
from wagtail_factories import PageFactory

import wagtaildraftsharing.models
from wagtaildraftsharing.models import WagtaildraftsharingLink

FROZEN_TIME_ISOFORMATTED = "2024-01-02 12:34:56.123456+00:00"


class TestWagtaildraftsharingLinkManager(TestCase):
    def setUp(self):
        self.test_user = User.objects.create(
            username="test", email="testuser@example.com"
        )

    def create_revision(self):
        page = PageFactory()

        # create the first revision
        page.save_revision().publish()

        old_title = page.title
        new_title = f"New {old_title}"
        page.title = new_title

        # create the second revision with a new title
        page.save_revision().publish()

        page.refresh_from_db()
        earliest_revision = page.revisions.earliest("created_at")
        return earliest_revision

    @freeze_time(FROZEN_TIME_ISOFORMATTED)
    def test_create_sharing_link__max_ttl_from_settings(self):
        frozen_time = datetime.datetime.fromisoformat(FROZEN_TIME_ISOFORMATTED)

        # Ensure we've got a level playing field: that the time is TZ-aware
        if not is_aware(frozen_time):
            self.fail("frozen_time was a naive datetime but it should not be")

        max_ttls_and_expected_expiries = (
            (300, frozen_time + datetime.timedelta(seconds=300)),
            (1250000, frozen_time + datetime.timedelta(seconds=1250000)),
            (-1, None),
        )

        for max_ttl, expected_expiry in max_ttls_and_expected_expiries:
            for method_name in [
                "get_or_create_for_revision",
                "create_for_revision",
            ]:
                with self.subTest(
                    max_ttl=max_ttl,
                    expected_expiry=expected_expiry,
                    method_name=method_name,
                ):
                    with patch.object(
                        wagtaildraftsharing.models.draftsharing_settings,
                        "MAX_TTL",
                        max_ttl,
                    ):
                        revision = self.create_revision()

                        link = getattr(WagtaildraftsharingLink.objects, method_name)(
                            revision=revision,
                            user=self.test_user,
                        )

                        assert link.active_until == expected_expiry, (
                            link.active_until,
                            expected_expiry,
                        )

    @freeze_time(FROZEN_TIME_ISOFORMATTED)
    def test_create_sharing_link__max_ttl_from_params(self):
        frozen_time = datetime.datetime.fromisoformat(FROZEN_TIME_ISOFORMATTED)

        # Ensure we've got a level playing field: that the time is TZ-aware
        if not is_aware(frozen_time):
            self.fail("frozen_time was a naive datetime but it should not be")

        max_ttls_and_expected_expiries = (
            (600, frozen_time + datetime.timedelta(seconds=600)),
            (250000, frozen_time + datetime.timedelta(seconds=250000)),
            (-1, None),
        )

        for max_ttl, expected_expiry in max_ttls_and_expected_expiries:
            for method_name in [
                "get_or_create_for_revision",
                "create_for_revision",
            ]:
                with self.subTest(
                    max_ttl=max_ttl,
                    expected_expiry=expected_expiry,
                    method_name=method_name,
                ):
                    revision = self.create_revision()

                    link = getattr(WagtaildraftsharingLink.objects, method_name)(
                        revision=revision, user=self.test_user, max_ttl=max_ttl
                    )

                    assert link.active_until == expected_expiry, (
                        link.active_until,
                        expected_expiry,
                    )

    def test_create_sharing_link_allows_multiple_links_per_revision(self):
        for method_name in [
            "get_or_create_for_revision",
            "create_for_revision",
        ]:
            with self.subTest(method_name=method_name):
                revision = self.create_revision()
                link_1 = getattr(WagtaildraftsharingLink.objects, method_name)(
                    revision=revision, user=self.test_user
                )
                link_2 = getattr(WagtaildraftsharingLink.objects, method_name)(
                    revision=revision, user=self.test_user
                )
                if method_name == "get_or_create_for_revision":
                    assert link_1.key == link_2.key
                else:
                    assert link_1.key != link_2.key


class TestWagtaildraftsharingLinkModel(TestCase):
    def create_revision(self):
        page = PageFactory()

        # create the first revision
        page.save_revision().publish()

        old_title = page.title
        new_title = f"New {old_title}"
        page.title = new_title

        # create the second revision with a new title
        page.save_revision().publish()

        page.refresh_from_db()
        earliest_revision = page.revisions.earliest("created_at")
        return earliest_revision

    def test_str_method(self):
        # Just chasing 100% coverage
        revision = self.create_revision()
        link = WagtaildraftsharingLink.objects.create(
            revision=revision,
        )
        self.assertEqual(str(link), f"Revision {revision.id} of New Test page")

    def test_url_method(self):
        link = WagtaildraftsharingLink.objects.create(
            revision=self.create_revision(),
        )
        expected_url = f"/wagtaildraftsharing/{link.key}/"
        self.assertEqual(link.url, expected_url)

    @patch.object(wagtail, "__version__", "5.0.0")  # only first digit matters
    def test_share_url_method__wagtail_5(self):
        link = WagtaildraftsharingLink.objects.create(
            revision=self.create_revision(),
        )
        expected = dedent(
            f"""<a
                class="button button-secondary button-small"
                data-wagtaildraftsharing-url
                target="_blank"
                rel="noopener noreferrer"
                href="/wagtaildraftsharing/{link.key}/">View</a>"""
        )
        self.assertEqual(dedent(link.share_url), expected)

    @patch.object(wagtail, "__version__", "6.0.0")  # only first digit matters
    def test_share_url_method__wagtail_6(self):
        link = WagtaildraftsharingLink.objects.create(
            revision=self.create_revision(),
        )
        expected = dedent(
            f"""<a
                class="button button-secondary button-small"
                data-controller="wagtaildraftsharing"
                data-wagtaildraftsharing-snippet-url
                target="_blank"
                rel="noopener noreferrer"
                href="/wagtaildraftsharing/{link.key}/">View</a>"""
        )
        self.assertEqual(dedent(link.share_url), expected)



# Generated by Django 4.2.16 on 2024-10-01 14:32

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("wagtaildraftsharing", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="wagtaildraftsharinglink",
            name="created_by",
            field=models.ForeignKey(
                blank=True,
                editable=False,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]



# Generated by Django 4.1.12 on 2023-11-26 09:48

import uuid

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('wagtailcore', '0089_log_entry_data_json_null_to_object'),
    ]

    operations = [
        migrations.CreateModel(
            name='WagtaildraftsharingLink',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('is_active', models.BooleanField(default=True)),
                ('active_until', models.DateTimeField(blank=True, help_text='Leave blank for no expiry', null=True)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('revision', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='wagtailcore.revision')),
            ],
            options={
                'verbose_name': 'Sharing Link',
                'verbose_name_plural': 'Sharing Links',
            },
        ),
    ]



import os

PROJECT_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
BASE_DIR = os.path.dirname(PROJECT_DIR)

DEBUG = True

ALLOWED_HOSTS = ["*"]

SECRET_KEY = "not needed"

ROOT_URLCONF = "wagtaildraftsharing.tests.urls"

DATABASES = {
    "default": {
        "ENGINE": os.environ.get("DATABASE_ENGINE", "django.db.backends.sqlite3"),
        "NAME": os.environ.get("DATABASE_NAME", "wagtaildraftsharing.sqlite"),
        "USER": os.environ.get("DATABASE_USER", None),
        "PASSWORD": os.environ.get("DATABASE_PASS", None),
        "HOST": os.environ.get("DATABASE_HOST", None),
        "TEST": {"NAME": os.environ.get("DATABASE_NAME", None)},
    },
}

WAGTAIL_APPS = (
    "wagtail.contrib.forms",
    "wagtail.contrib.routable_page",
    "wagtail.contrib.settings",
    "wagtail.test.routablepage",
    "wagtail.admin",
    "wagtail",
    "wagtail.documents",
    "wagtail.images",
    "wagtail.sites",
    "wagtail.snippets",
    "wagtail.users",
)

MIDDLEWARE = (
    "django.middleware.common.CommonMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
)

INSTALLED_APPS = (
    (
        "django.contrib.admin",
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.messages",
        "django.contrib.sessions",
        "django.contrib.staticfiles",
        "taggit",
    )
    + WAGTAIL_APPS
    + ("wagtaildraftsharing",)
)

STATIC_ROOT = os.path.join(BASE_DIR, "static")
STATIC_URL = "/static/"

MEDIA_ROOT = os.path.join(BASE_DIR, "media")
MEDIA_URL = "/media/"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
                "django.template.context_processors.request",
            ],
            "debug": True,
        },
    },
]

WAGTAIL_SITE_NAME = "Test Site"

WAGTAILADMIN_BASE_URL = "http://localhost:8000"

USE_TZ = True

# Wagtaildraftsharing settings

WAGTAILDRAFTSHARING = {
    # Here is where settings would be overridden in a real project
    # "MAX_TTL": 12312321,
}



# Generated by Django 5.1.1 on 2024-12-04 10:56

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("wagtaildraftsharing", "0002_alter_wagtaildraftsharinglink_created_by"),
    ]

    operations = [
        migrations.AlterField(
            model_name="wagtaildraftsharinglink",
            name="revision",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="+",
                to="wagtailcore.revision",
            ),
        ),
    ]



import datetime
import json

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.http import Http404
from django.test import RequestFactory, TestCase
from django.urls import reverse
from django.utils.timezone import now as timezone_now
from wagtail_factories import PageFactory

from wagtaildraftsharing.models import WagtaildraftsharingLink
from wagtaildraftsharing.views import CreateSharingLinkView, SharingLinkView

User = get_user_model()

FROZEN_TIME_ISOFORMATTED = "2024-01-02 12:34:56.123456+00:00"


class CreateSharingLinkViewTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.superuser = User.objects.create_superuser(username="admin", password="test")
        cls.factory = RequestFactory()

    def create_revision(self):
        page = PageFactory()

        # create the first revision
        page.save_revision().publish()

        old_title = page.title
        new_title = f"New {old_title}"
        page.title = new_title

        # create the second revision with a new title
        page.save_revision().publish()

        page.refresh_from_db()
        earliest_revision = page.revisions.earliest("created_at")
        return earliest_revision

    def test_create_sharing_link_view(self):
        revision = self.create_revision()
        request = self.factory.post("/create/", {"revision": revision.id})
        request.user = self.superuser

        response = CreateSharingLinkView.as_view()(request)
        self.assertEqual(response.status_code, 200)

        response_data = json.loads(response.content.decode("utf-8"))
        self.assertEqual(
            response_data["url"], WagtaildraftsharingLink.objects.get().url
        )

    def test_create_sharing_link_view__invalid_data(self):
        revision = self.create_revision()
        request = self.factory.post("/create/", {"revision": revision.id + 123})
        request.user = self.superuser

        self.assertEqual(WagtaildraftsharingLink.objects.count(), 0)
        response = CreateSharingLinkView.as_view()(request)
        self.assertEqual(response.status_code, 400)
        self.assertEqual(WagtaildraftsharingLink.objects.count(), 0)

    def test_create_sharing_link_view__editor_allowed(self):
        # Assumes default Wagtail staff-side user groups are present
        editor_group = Group.objects.get(name="Editors")
        editor = User.objects.create_user(username="editor", password="test")
        editor.groups.add(editor_group)

        revision = self.create_revision()
        dest = reverse("wagtaildraftsharing:create")
        self.assertEqual(WagtaildraftsharingLink.objects.count(), 0)
        self.client.login(username="editor", password="test")
        response = self.client.post(dest, data={"revision": revision.id}, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(WagtaildraftsharingLink.objects.count(), 1)

    def test_create_sharing_link_view__moderator_allowed(self):
        moderator_group = Group.objects.get(name="Moderators")
        moderator = User.objects.create_user(username="moderator", password="test")
        moderator.groups.add(moderator_group)

        revision = self.create_revision()
        dest = reverse("wagtaildraftsharing:create")
        self.assertEqual(WagtaildraftsharingLink.objects.count(), 0)
        self.client.login(username="moderator", password="test")
        response = self.client.post(dest, data={"revision": revision.id}, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(WagtaildraftsharingLink.objects.count(), 1)

    def test_create_sharing_link_view__anonymous_user_not_allowed(self):
        revision = self.create_revision()
        dest = reverse("wagtaildraftsharing:create")
        self.assertEqual(WagtaildraftsharingLink.objects.count(), 0)
        response = self.client.post(dest, data={"revision": revision.id}, follow=True)
        self.assertContains(
            response,
            "Sorry, you do not have permission to access this area",
        )
        self.assertEqual(WagtaildraftsharingLink.objects.count(), 0)


class SharingLinkViewTests(TestCase):
    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.superuser = User.objects.create_superuser(username="admin", password="test")
        cls.factory = RequestFactory()

    def create_revision(self):
        page = PageFactory()

        # create the first revision
        page.save_revision().publish()

        old_title = page.title
        new_title = f"New {old_title}"
        page.title = new_title

        # create the second revision with a new title
        page.save_revision().publish()

        page.refresh_from_db()
        earliest_revision = page.revisions.earliest("created_at")
        return earliest_revision

    def test_sharing_link_view__valid_link(self):
        revision = self.create_revision()
        sharing_link = WagtaildraftsharingLink.objects.create(
            revision=revision, created_by=self.superuser
        )

        request = self.factory.get(f"/{sharing_link.key}/")
        response = SharingLinkView.as_view()(request, key=sharing_link.key)

        self.assertEqual(response.status_code, 200)
        self.assertContains(response, f"<title>{revision.as_object().title}</title>")

    def test_sharing_link_view__invalid_link_404s(self):
        revision = self.create_revision()
        sharing_link = WagtaildraftsharingLink.objects.create(
            revision=revision, created_by=self.superuser
        )

        dummy_key = "336611cb-c2aa-4e2b-9b52-1d183939cbf8"
        assert sharing_link.key != dummy_key  # Extremely unlikely to clash!

        request = self.factory.get(f"/{dummy_key}/")
        with self.assertRaises(Http404):
            SharingLinkView.as_view()(request, key=dummy_key)

    def test_sharing_link_view__expired_link_404s_and_sets_is_active_to_False(self):
        revision = self.create_revision()
        sharing_link = WagtaildraftsharingLink.objects.create(
            revision=revision, created_by=self.superuser
        )
        request = self.factory.get(f"/{sharing_link.key}/")
        assert sharing_link.is_active is True

        response = SharingLinkView.as_view()(request, key=sharing_link.key)
        self.assertEqual(response.status_code, 200)

        sharing_link.active_until = timezone_now() - datetime.timedelta(seconds=1)
        sharing_link.save()
        sharing_link.refresh_from_db()

        assert sharing_link.is_active is True  # still true until it's called

        with self.assertRaises(Http404):
            SharingLinkView.as_view()(request, key=sharing_link.key)

        sharing_link.refresh_from_db()
        assert sharing_link.is_active is False  # calling it after expiry deactivates it

    def test_sharing_link_view__inactive_link_404s(self):
        revision = self.create_revision()
        sharing_link = WagtaildraftsharingLink.objects.create(
            revision=revision, created_by=self.superuser, is_active=False
        )
        request = self.factory.get(f"/{sharing_link.key}/")
        with self.assertRaises(Http404):
            SharingLinkView.as_view()(request, key=sharing_link.key)

    def test_sharing_link_includes_robots_header(self):
        revision = self.create_revision()
        sharing_link = WagtaildraftsharingLink.objects.create(
            revision=revision, created_by=self.superuser
        )

        request = self.factory.get(f"/{sharing_link.key}/")
        response = SharingLinkView.as_view()(request, key=sharing_link.key)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response["X-Robots-Tag"], "noindex, nofollow")



from django.http import Http404, JsonResponse
from django.shortcuts import get_object_or_404
from django.utils.decorators import method_decorator
from django.views.generic import CreateView
from wagtail.admin.auth import user_has_any_page_permission, user_passes_test
from wagtail.admin.views.generic.preview import PreviewRevision
from wagtail.models import Page, Revision

from wagtaildraftsharing.forms import CreateWagtaildraftsharingLinkForm
from wagtaildraftsharing.models import WagtaildraftsharingLink
from wagtaildraftsharing.utils import tz_aware_utc_now


class SharingLinkView(PreviewRevision):
    def setup(self, request, *args, **kwargs):
        key = kwargs.pop("key")
        now = tz_aware_utc_now()

        sharing_link = get_object_or_404(
            WagtaildraftsharingLink,
            key=key,
            is_active=True,
        )
        if sharing_link.active_until and sharing_link.active_until < now:
            sharing_link.is_active = False
            sharing_link.save(update_fields=["is_active"])
            raise Http404

        revision_id = sharing_link.revision_id
        page_id = get_object_or_404(Revision, id=revision_id).object_id
        return super().setup(request, *args, page_id, revision_id, **kwargs)

    def get_object(self):
        return get_object_or_404(Page, id=self.pk).specific

    def get(self, request, *args, **kwargs):
        response = super().get(request, *args, **kwargs)
        response["X-Robots-Tag"] = "noindex, nofollow"
        return response


@method_decorator(user_passes_test(user_has_any_page_permission), name="dispatch")
class CreateSharingLinkView(CreateView):
    model = WagtaildraftsharingLink
    form_class = CreateWagtaildraftsharingLinkForm

    def form_valid(self, form):
        sharing_link = WagtaildraftsharingLink.objects.get_or_create_for_revision(
            revision=form.cleaned_data["revision"],
            user=self.request.user,
        )
        return JsonResponse({"url": sharing_link.url})

    def form_invalid(self, form):
        return JsonResponse({"errors": form.errors}, status=400)



from django.urls import path

from wagtaildraftsharing.views import CreateSharingLinkView, SharingLinkView


urlpatterns = [
    path("<uuid:key>/", SharingLinkView.as_view(), name="view"),
    path("create/", CreateSharingLinkView.as_view(), name="create"),
]

app_name = "wagtaildraftsharing"



from django.utils.translation import gettext_lazy as _
from wagtail.log_actions import LogFormatter

WAGTAILDRAFTSHARING_CREATE_SHARING_LINK = "wagtaildraftsharing.create_sharing_link"
WAGTAILDRAFTSHARING_REUSE_SHARING_LINK = "wagtaildraftsharing.reuse_sharing_link"


def register_wagtaildraftsharing_log_actions(actions):
    @actions.register_action(WAGTAILDRAFTSHARING_CREATE_SHARING_LINK)
    class CreateSharingLink(LogFormatter):
        label = _("Create sharing link")

        def format_message(self, log_entry):
            return _(
                "{username} created sharing link for revision {revision_id}"
            ).format(
                revision_id=log_entry.data["revision"],
                username=log_entry.user,
            )

    @actions.register_action(WAGTAILDRAFTSHARING_REUSE_SHARING_LINK)
    class ReuseSharingLink(LogFormatter):
        label = _("Reuse sharing link")

        def format_message(self, log_entry):
            return _(
                "{username} reused sharing link for revision {revision_id}"
            ).format(
                revision_id=log_entry.data["revision"],
                username=log_entry.user,
            )
